#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/power.h>
#include "ATmega88PA.h"
#include "device.h"

void DEVICE_Init(void) {
    //Outputs
    DDRB |= LE;
    DDRC |= A1|A2|A3|A4|A5;
    DDRD |= OE|SDI|CLK|LED|BUZZER;
    //Input pullups
    PORTB |= SW1|SW2|SW3|SW4|IR;
    PORTD |= RXD|TXD;
}

void DEVICE_Sleep(void) {
    ADC_DISABLE;
    //OE can leak power through ESD diodes
    //temporary change OE to input
    DDRD &= ~OE;
    //set pins low
    PORTB &= ~(LE|IR);
    PORTC &= ~(A1|A2|A3|A4|A5);
    PORTD &= ~(OE|SDI|CLK|LED|BUZZER|RXD|TXD);
    //slow down to 1 MHz
    clock_prescale_set(clock_div_8);
    //go to sleep
    set_sleep_mode(SLEEP_MODE_PWR_SAVE);
    sleep_mode();
}

void DEVICE_Wakeup(void) {
    ADC_ENABLE;
    //restore OE as output
    DDRD |= OE;
    //restore input pullups
    PORTB |= SW1|SW2|SW3|SW4|IR;
    PORTD |= RXD|TXD;
    //speed up to 8 MHz
    clock_prescale_set(clock_div_1);
}
