#include <avr/io.h>
#include <avr/interrupt.h>
#include "ATmega88PA.h"
#include "device.h"
#include "rtc.h"
#include "stub.h"
#include "ir_rc5.h"
#include "ir_nec.h"
#include "ir_kas.h"
#include "ir.h"

IR_PACKET packet = {IR_UNKNOWN, 0, 0};

void IR_Init(void) {
    TIMER1_CLEAR_TIMER_ON_COMPARE_MATCH;
    OCR1A = 15000;
    TIMER1_CLEAR_COMPARE_MATCH_A_INTERRUPT_FLAG;
    TIMER1_COMPARE_MATCH_A_INTERRUPT_ENABLE;
    TIMER1_INPUT_CAPTURE_ON_FALLING_EDGE;
    TIMER1_ACTIVATE_INPUT_CAPTURE_NOISE_CANCELER;
    TIMER1_INPUT_CAPTURE_INTERRUPT_DISABLE;
    IR_SignalAppear = STUB_void;
    IR_SignalDisappear = STUB_void;
}

void IR_Activate(void) {
    TIMER1_INPUT_CAPTURE_INTERRUPT_ENABLE;
}

static void IR_Reset(void) {
    packet.protocol = IR_UNKNOWN;
}

ISR(TIMER1_CAPT_vect) {
    IR_SignalAppear();
    uint16_t delay = ICR1;
    TIMER1_SET_VALUE(TCNT1 - ICR1);
    TIMER1_TOGGLE_INPUT_CAPTURE_TRIGGER_EDGE;
    TIMER1_CLEAR_INPUT_CAPTURE_INTERRUPT_FLAG;
    TIMER1_CLOCK_PRESCALER_8_START;
    if(RC5_Decode(delay)) {
        packet.protocol = IR_RC5;
        packet.address = RC5_Address();
        packet.command = RC5_Command();
    } else if(NEC_Decode(delay)) {
        packet.protocol = IR_NEC;
        packet.address = NEC_Address();
        packet.command = NEC_Command();
    } else if(KAS_Decode(delay)) {
        packet.protocol = IR_KAS;
        packet.address = KAS_Address();
        packet.command = KAS_Command();
    }
}

ISR(TIMER1_COMPA_vect) {
    TIMER1_CLOCK_PRESCALER_0_STOP;
    TIMER1_RESET;
    TIMER1_INPUT_CAPTURE_ON_FALLING_EDGE;
    TIMER1_CLEAR_INPUT_CAPTURE_INTERRUPT_FLAG;
    IR_SignalDisappear();
    if(packet.protocol != IR_UNKNOWN) {
        IR_PacketReceived(packet);
    }
    IR_Reset();
    RC5_Reset();
    NEC_Reset();
    KAS_Reset();
}
