#include <avr/io.h>
#include "ATmega88PA.h"
#include "device.h"
#include "display.h"
#include "bcd.h"
#include "ir_kas.h"

/* Kaseikyo Protocol Decoding */

// microseconds
#define START_PULSE_MIN 3000
#define START_PULSE_MAX 4000
#define START_SPACE_MIN 1500
#define START_SPACE_MAX 2000
#define SHORT_MIN 216
#define SHORT_MAX 648
#define LONG_MIN  864
#define LONG_MAX 1728

typedef enum {
    START_PULSE,
    START_SPACE,
    BIT_PULSE,
    BIT_SPACE
} KAS_STATE;

static uint32_t kas, data;
static int8_t counter;
static KAS_STATE state = START_PULSE;

#define EMMIT_0 counter++
#define EMMIT_1 if(counter > 0) { kas |= (1UL<<counter); } counter++

void KAS_Reset(void) {
    counter = -16;
    kas = 0;
    state = START_PULSE;
}

uint8_t KAS_Address(void) {
    return data;
}

uint8_t KAS_Command(void) {
    return data>>16UL;
}

uint8_t KAS_Decode(uint16_t delay) {
    switch(state) {
        case BIT_PULSE:
            if(counter == 32) {
                data = kas;
                KAS_Reset();
                return 1;
            }
            state = BIT_SPACE;
            if((delay < SHORT_MIN) || (delay > SHORT_MAX)) {
                KAS_Reset();
            }
            return 0;
        case BIT_SPACE:
            state = BIT_PULSE;
            if((delay > SHORT_MIN) && (delay < SHORT_MAX)) {
                EMMIT_0;
            } else if((delay > LONG_MIN) && (delay < LONG_MAX)) {
                EMMIT_1;
            } else { KAS_Reset(); }
            return 0;
        case START_PULSE:
            state = START_SPACE;
            if((delay < START_PULSE_MIN) || (delay > START_PULSE_MAX)) {
                KAS_Reset();
            }
            return 0;
        case START_SPACE:
            state = BIT_PULSE;
            data = 0;
            if((delay < START_SPACE_MIN) || (delay > START_SPACE_MAX)) {
                KAS_Reset();
            }
            return 0;
    }
    return 0;
}
